"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MidwayKafkaFramework = void 0;
const core_1 = require("@midwayjs/core");
const decorator_1 = require("@midwayjs/decorator");
const kafka_1 = require("./kafka");
let MidwayKafkaFramework = class MidwayKafkaFramework extends core_1.BaseFramework {
    configure() {
        return this.configService.getConfiguration('kafka');
    }
    async applicationInitialize() {
        // Create a connection manager
        this.app = new kafka_1.KafkaConsumerServer({
            logger: this.logger,
            ...this.configurationOptions,
        });
    }
    async run() {
        try {
            await this.app.connect(this.configurationOptions.kafkaConfig, this.configurationOptions.consumerConfig);
            await this.loadSubscriber();
            this.logger.info('Kafka consumer server start success');
        }
        catch (error) {
            this.logger.error('Kafka consumer connect fail', error);
            await this.app.closeConnection();
        }
    }
    async loadSubscriber() {
        const subscriberModules = (0, decorator_1.listModule)(decorator_1.MS_CONSUMER_KEY, module => {
            const metadata = (0, decorator_1.getClassMetadata)(decorator_1.MS_CONSUMER_KEY, module);
            return metadata.type === decorator_1.MSListenerType.KAFKA;
        });
        for (const module of subscriberModules) {
            const data = (0, decorator_1.listPropertyDataFromClass)(decorator_1.MS_CONSUMER_KEY, module);
            const topicTitles = [...new Set(data.map(e => e[0].topic))];
            const midwayConsumerConfigs = topicTitles.map(e => {
                const midwayConsumerConfig = {
                    topic: e,
                    subscription: {},
                    runConfig: {},
                };
                const comsumerParams = data
                    .map(value => {
                    if (value[0].topic === midwayConsumerConfig.topic) {
                        return Object.assign(midwayConsumerConfig, value[0]);
                    }
                })
                    .filter(e => e && true);
                if (comsumerParams && Object.keys(comsumerParams[0]).length > 0) {
                    return comsumerParams[0];
                }
                return midwayConsumerConfig;
            });
            midwayConsumerConfigs.forEach(async (e) => {
                await this.app.connection.subscribe(Object.assign({
                    topics: topicTitles,
                }, e.subscription));
                await this.app.connection.run(Object.assign(e.runConfig, {
                    eachMessage: async ({ topic, partition, message }) => {
                        let propertyKey;
                        for (const methodBindListeners of data) {
                            for (const listenerOptions of methodBindListeners) {
                                if (topic === listenerOptions.topic) {
                                    propertyKey = listenerOptions.propertyKey;
                                    const ctx = {
                                        topic: topic,
                                        partition: partition,
                                        message: message,
                                        commitOffsets: (offset) => {
                                            return this.app.connection.commitOffsets([
                                                {
                                                    topic: topic,
                                                    partition: partition,
                                                    offset,
                                                },
                                            ]);
                                        },
                                    };
                                    this.app.createAnonymousContext(ctx);
                                    const ins = await ctx.requestContext.getAsync(module);
                                    const fn = await this.applyMiddleware(async () => {
                                        return await ins[propertyKey].call(ins, message);
                                    });
                                    try {
                                        const result = await fn(ctx);
                                        // 返回为undefined，下面永远不会执行
                                        if (result) {
                                            const res = await this.app.connection.commitOffsets(message.offset);
                                            return res;
                                        }
                                    }
                                    catch (error) {
                                        // 记录偏移量提交的异常情况
                                        this.logger.error(error);
                                    }
                                }
                            }
                        }
                    },
                }));
            });
        }
    }
    getFrameworkName() {
        return 'midway:kafka';
    }
    async beforeStop() {
        await this.app.close();
    }
    getFrameworkType() {
        return decorator_1.MidwayFrameworkType.MS_KAFKA;
    }
};
MidwayKafkaFramework = __decorate([
    (0, decorator_1.Framework)()
], MidwayKafkaFramework);
exports.MidwayKafkaFramework = MidwayKafkaFramework;
//# sourceMappingURL=framework.js.map